@@ -93,3 +93,7 @@ gboolean on_monitor_win_delete_event(GtkWidget * widget, GdkEvent * event, gpoin
 void on_help_close_button_clicked(GtkButton * button, gpointer user_data);
 
 #endif /* _CALLBACKS_H_ */
+
+void
+on_monitor_seekeof_button_clicked      (GtkButton       *button,
+                                        gpointer         user_data);
