@@ -27,14 +27,23 @@
 #endif
 
 #include "crossfade.h"
-#include "configure.h"
-#include "monitor.h"
+#include "cfgutil.h"
 #include "effect.h"
 #include "format.h"
 #include "convert.h"
 #include "rate.h"
 #include "volume.h"
 #include "timing.h"
+
+#ifdef HAVE_GTK
+#  include "configure.h"
+#  include "monitor.h"
+#else
+void xfade_check_monitor_win() { }
+void xfade_start_monitor() { }
+void xfade_stop_monitor() { }
+#endif
+
 #ifdef HAVE_GTK_2_0
 #  include "interface-2.0.h"
 #  include "support-2.0.h"
@@ -42,6 +51,7 @@
 #  include "interface.h"
 #  include "support.h"
 #endif
+
 #ifdef HAVE_OSS
 #  include "oss.h"
 #endif
@@ -68,6 +78,7 @@
 
 /* output plugin callback prototypes */
 static void xfade_init();
+static void xfade_cleanup();  /* audacious and patched only */
 static void xfade_set_volume(int l, int r);
 static void xfade_get_volume(int *l, int *r);
 static gint xfade_open_audio(AFormat fmt, int rate, int nch);
@@ -80,27 +91,53 @@ static gint xfade_buffer_playing();
 static gint xfade_written_time();
 static gint xfade_output_time();
 
-/* output plugin callback table */
-static OutputPlugin xfade_op = {
-	NULL,
-	NULL,
-	"Crossfade Plugin " VERSION,
-	xfade_init,
-	xfade_about,
-	xfade_configure,
-	xfade_get_volume,
-	xfade_set_volume,
-	xfade_open_audio,
-	xfade_write_audio,
-	xfade_close_audio,
-	xfade_flush,
-	xfade_pause,
-	xfade_buffer_free,
-	xfade_buffer_playing,
-	xfade_output_time,
-	xfade_written_time
+/* output plugin callback table (extended, needs patched player) */
+static struct
+{
+	OutputPlugin xfade_op;
+        void (*cleanup) (void);
+}
+xfade_op_private =
+{
+	{
+		NULL,
+		NULL,
+		"Crossfade Plugin " VERSION,
+		xfade_init,
+#if defined(COMPILE_FOR_AUDACIOUS)
+		xfade_cleanup,
+#endif
+#if defined(HAVE_GTK)
+		xfade_about,
+		xfade_configure,
+#else
+		NULL,
+		NULL,
+#endif
+		xfade_get_volume,
+		xfade_set_volume,
+		xfade_open_audio,
+		xfade_write_audio,
+		xfade_close_audio,
+		xfade_flush,
+		xfade_pause,
+		xfade_buffer_free,
+		xfade_buffer_playing,
+		xfade_output_time,
+		xfade_written_time,
+#if defined(COMPILE_FOR_AUDACIOUS)
+		NULL
+	},
+	NULL
+#else
+	},
+	xfade_cleanup
+#endif
 };
 
+static OutputPlugin *xfade_op = &xfade_op_private.xfade_op;
+
+
 /* internal prototypes */
 static void load_symbols();
 static void output_list_hack();
@@ -113,6 +150,7 @@ static void sync_output();
 static gboolean *xmms_playlist_get_info_going = NULL;   /* XMMS */
 static gboolean *xmms_is_quitting             = NULL;   /* XMMS */
 static gboolean *input_stopped_for_restart    = NULL;   /* XMMS */
+static char *  (*playlist_get_fadeinfo)(int)  = NULL;   /* XMMS patch */
 
 #if defined(COMPILE_FOR_BMP) || defined(COMPILE_FOR_AUDACIOUS)
 gboolean bmp_playback_get_playing();  /* BMP */
@@ -122,76 +160,6 @@ gboolean get_input_playing()
 }
 #endif
 
-/* TEST: test area for multithreading issues */
-#if 0
-static gboolean  test_thread_running_ack = FALSE;
-static gboolean  test_thread_running = TRUE;
-static pthread_t test_thread;
-
-#define USE_PTHREAD_JOIN
-void *test_thread_f(void *arg)
-{
-	fprintf(stderr, "*** TEST THREAD BEGIN\n");
-
-	while (test_thread_running)
-	{
-		fprintf(stderr, "*\n");
-		xmms_usleep(1000*1000);
-	}
-
-	fprintf(stderr, "*** TEST THREAD END\n");
-	test_thread_running_ack = TRUE;
-#ifdef USE_PTHREAD_JOIN
-	pthread_exit(0);
-#endif
-	return NULL;
-}
-
-void test_fini();
-void test_init() __attribute__((constructor));
-void test_init()
-{
-	fprintf(stderr, "*** TEST: creating thread...\n");
-	
-	pthread_attr_t attr;
-	pthread_attr_init(&attr);
-	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-
-#ifdef USE_PTHREAD_JOIN
-	if (pthread_create(&test_thread, NULL, test_thread_f, NULL))
-#else
-	if (pthread_create(&test_thread, &attr, test_thread_f, NULL))
-#endif
-		perror("*** TEST: create()");
-		
-	pthread_attr_destroy(&attr);
-#ifdef _POSIX_PRIORITY_SCHEDULING 
-	sched_yield();
-#endif
-
-	fprintf(stderr, "*** TEST: creating thread... done\n");
-	
-//	test_fini();
-}
-
-void test_fini() __attribute__((destructor));
-void test_fini()
-{
-	fprintf(stderr, "*** TEST: joining...\n");
-
-	test_thread_running = FALSE;
-#ifdef USE_PTHREAD_JOIN
-	if (pthread_join(test_thread, NULL))
-		perror("*** TEST: join()");
-#else
-	while (!test_thread_running_ack)
-		xmms_usleep(10000);
-#endif
-		
-	fprintf(stderr, "*** TEST: joining... done\n");
-}
-#endif
-
 #ifdef HAVE_CTOR
 void fini() __attribute__ ((destructor));
 #else
@@ -224,7 +192,7 @@ static plugin_config_t the_op_config = DEFAULT_OP_CONFIG;
 static gboolean input_playing = FALSE;
 
        gboolean output_opened     = FALSE;
-       gboolean output_restart    = FALSE;
+       gboolean output_restart    = FALSE;  /* used by XMMS 'songchange' patch */
 static gint     output_flush_time = 0;
        gint     output_offset     = 0;
 static gint64   output_written    = 0;
@@ -237,6 +205,23 @@ static quantaudio_t qa, last_qa;
 static gboolean have_qa = FALSE, have_last_qa = FALSE;
 #endif
 
+#ifdef TIMING_COMMENTS
+typedef struct
+{
+	gboolean enable;
+	gint     len_ms, volume, skip_ms, ofs_ms;
+}
+timing_half_config_t;
+
+typedef struct
+{
+	timing_half_config_t in;
+	timing_half_config_t out;
+}
+timing_config_t;
+
+static timing_config_t last_timing, current_timing;
+#endif
 
 /*
  *  Available fade configs:
@@ -307,13 +292,13 @@ static fade_config_t *fade_config = NULL;
 OutputPlugin *
 get_oplugin_info()
 {
-	return &xfade_op;
+	return xfade_op;
 }
 
 OutputPlugin *
 get_crossfade_oplugin_info()
 {
-	return &xfade_op;
+	return xfade_op;
 }
 
 static gint
@@ -355,6 +340,7 @@ xfade_realize_config()		/* also called by xfade_init() */
 
 #ifdef VOLUME_NORMALIZER
 	/* update volume normalizer target rms */
+
 	volume_set_target_rms(&volume_context, config->volnorm_target);
 #endif
 
@@ -409,7 +395,7 @@ find_output()
 			if ((element = g_list_find_custom(list, config->op_name, output_list_f)))
 				op = element->data;
 
-		if (op == &xfade_op)
+		if (op == xfade_op)
 		{
 			DEBUG(("[crossfade] find_output: can't use myself as output plugin!\n"));
 			op = NULL;
@@ -619,14 +605,18 @@ load_symbols()
 	input_stopped_for_restart = dlsym(handle, "input_stopped_for_restart");
 	DEBUG((!(error = dlerror())? " found\n" : " missing\n"));
 
-	DEBUG(("[crossfade] load_symbols: playlist_get_info_going:"));
+	/* DEBUG(("[crossfade] load_symbols: playlist_get_info_going:")); */
 	xmms_playlist_get_info_going = dlsym(handle, "playlist_get_info_going");
-	DEBUG((!(error = dlerror())? " found\n" : " missing\n"));
+	/* DEBUG((!(error = dlerror())? " found\n" : " missing\n")); */
 
 	DEBUG(("[crossfade] load_symbols: is_quitting:"));
 	xmms_is_quitting = dlsym(handle, "is_quitting");
 	DEBUG((!(error = dlerror())? " found\n" : " missing\n"));
 
+	DEBUG(("[crossfade] load_symbols: playlist_get_fadeinfo:"));
+	playlist_get_fadeinfo = dlsym(handle, "playlist_get_fadeinfo");
+	DEBUG((!(error = dlerror())? " found\n" : " missing\n"));
+
 	dlclose(handle);
 #endif
 }
@@ -641,14 +631,14 @@ output_list_hack()
 {
 	GList *output_list = get_output_list();
 
-	int i0 = g_list_index(output_list, &xfade_op), i1;
+	int i0 = g_list_index(output_list, xfade_op), i1;
 
 	GList *first = g_list_first(output_list);
-	GList *xfade = g_list_find(output_list, &xfade_op);
+	GList *xfade = g_list_find(output_list, xfade_op);
 	xfade->data = first->data;
-	first->data = &xfade_op;
+	first->data = xfade_op;
 
-	i1 = g_list_index(output_list, &xfade_op);
+	i1 = g_list_index(output_list, xfade_op);
 	if (i0 != i1)
 		DEBUG(("[crossfade] output_list_hack: crossfade moved from index %d to %d\n", i0, i1));
 }
@@ -662,6 +652,20 @@ fini()
 	/* wait for buffer_thread to stop */
 	pthread_mutex_lock(&buffer_mutex);
 
+	/*
+	 * Special handling if the output is still opened. This can only happen
+	 * without the xmms 'exit' patch, which will allow us to detect if we
+	 * are about to exit when close_audio() is called.
+	 *
+	 * This may also happen if the output_keep_opened option is enabled. In
+	 * this case, we are not close_audio()ed because we are have never been
+	 * opened before.
+	 */
+	if (output_opened)
+	{
+		DEBUG(("[crossfade] fini: WARNING: output still opened, lockup imminent!\n"));
+	}
+
 	/* reset last_close so that the buffer thread will
 	 * not unnecessarily wait for a timeout. */
 	memset(&last_close, 0, sizeof(last_close));
@@ -685,6 +689,8 @@ fini()
 	 *
 	 * For yet unexplained reasons, pthread_join() will not work from
 	 * within code executed by a dynamic library's destructor code.
+	 *
+	 * http://groups.google.de/group/comp.programming.threads/msg/17b4c7a4f35bd1c5
 	 */
 #if 0
 	if (output_opened)
@@ -800,6 +806,8 @@ buffer_mfg_reset(buffer_t *buf, config_t *cfg)
 	buf->gap_len    = buf->gap;
 	buf->gap_level  = cfg->gap_lead_level;
 	buf->gap_killed = 0;
+	buf->skip       = 0;
+	buf->skip_len   = 0;
 }
 
 static void
@@ -822,10 +830,13 @@ buffer_reset(buffer_t *buf, config_t *cfg)
 static void
 xfade_apply_fade_config(fade_config_t *fc)
 {
-	gint avail, out_len, in_len, offset, skip;
+	gint out_skip, in_skip;
+	gint avail, out_len, in_len, offset, preload;
 	gint index, length, fade, n;
 	gfloat out_scale, in_scale;
-	gboolean out_len_clipped = FALSE, offset_clipped = FALSE;
+	gboolean out_skip_clipped = FALSE;
+	gboolean out_len_clipped = FALSE;
+	gboolean offset_clipped = FALSE;
 
 	/* Overwrites mix and fade; may add silence */
 
@@ -854,31 +865,51 @@ xfade_apply_fade_config(fade_config_t *fc)
 	 */
 
 	out_scale = 1.0f - (gfloat) xfade_cfg_fadeout_volume(fc) / 100.0f;
-	in_scale = 1.0f - (gfloat) xfade_cfg_fadein_volume(fc) / 100.0f;
+	in_scale  = 1.0f - (gfloat) xfade_cfg_fadein_volume (fc) / 100.0f;
 
 	/* rules (see above) */
-	/* a */
+	/* a: leave preload untouched */
 	avail = buffer->used - buffer->preload_size;
 	if (avail < 0)
 		avail = 0;
 
-	/* b */
+	/* skip end of song */
+	out_skip = MS2B(xfade_cfg_out_skip(fc)) & -4;
+	if (out_skip > avail)
+	{
+		DEBUG(("[crossfade] apply_fade_config: WARNING: clipping out_skip (%d -> %d)!\n", B2MS(out_skip), B2MS(avail)));
+		out_skip = avail;
+		out_skip_clipped = TRUE;
+	}
+
+	if (out_skip > 0)
+	{
+		buffer->used -= out_skip;
+		avail -= out_skip;
+	}
+
+	/* b: fadeout */
 	out_len = MS2B(xfade_cfg_fadeout_len(fc)) & -4;
-	if (out_len < 0)
-		out_len = 0;
 	if (out_len > avail)
 	{
 		DEBUG(("[crossfade] apply_fade_config: WARNING: clipping out_len (%d -> %d)!\n", B2MS(out_len), B2MS(avail)));
 		out_len = avail;
 		out_len_clipped = TRUE;
 	}
+	else if (out_len < 0)
+		out_len = 0;
 
-	/* c */
+	/* skip beginning of song */
+	in_skip = MS2B(xfade_cfg_in_skip(fc)) & -4;
+	if (in_skip < 0)
+		in_skip = 0;
+
+	/* c: fadein */
 	in_len = MS2B(xfade_cfg_fadein_len(fc)) & -4;
 	if (in_len < 0)
 		in_len = 0;
 
-	/* d */
+	/* d: offset (mixing point) */
 	offset = MS2B(xfade_cfg_offset(fc)) & -4;
 	if (offset < -avail)
 	{
@@ -890,9 +921,9 @@ xfade_apply_fade_config(fade_config_t *fc)
 		offset = buffer->mix_size - out_len;
 
 	/* e */
-	skip = buffer->preload_size;
-	if (skip > buffer->used)
-		skip = buffer->used;
+	preload = buffer->preload_size;
+	if (preload > buffer->used)
+		preload = buffer->used;
 
 	/* cut off rest of stream (decreases latency on manual songchange) */
 	if (fc->flush)
@@ -937,6 +968,15 @@ xfade_apply_fade_config(fade_config_t *fc)
 	/* Initialize fadein. Note that the actual fading / mixing will be done
 	 * on-the-fly when audio data is received by xfade_write_audio() */
 
+	/* start skipping */
+	if (in_skip > 0)
+	{
+		buffer->skip = in_skip;
+		buffer->skip_len = in_skip;
+	}
+	else
+		buffer->skip = 0;
+
 	/* start fading in */
 	if (in_len > 0)
 	{
@@ -969,8 +1009,10 @@ xfade_apply_fade_config(fade_config_t *fc)
 	}
 
 	/* done */
-	DEBUG(("[crossfade] apply_fade_config: avail=%d out=%d in=%d offset=%d skip=%d\n",
-	       B2MS(avail), B2MS(out_len), B2MS(in_len), B2MS(offset), B2MS(skip)));
+	if (in_skip || out_skip)
+		DEBUG(("[crossfade] apply_fade_config: out_skip=%d in_skip=%d\n", B2MS(out_skip), B2MS(in_skip)));
+	DEBUG(("[crossfade] apply_fade_config: avail=%d out=%d in=%d offset=%d preload=%d\n",
+	       B2MS(avail), B2MS(out_len), B2MS(in_len), B2MS(offset), B2MS(preload)));
 }
 
 static gint
@@ -1016,7 +1058,7 @@ album_match(gchar *old, gchar *new)
 
 	if (!same_dir)
 	{
-		DEBUG(("[crossfade] album_match: " "no match (different dirs)\n"));
+		DEBUG(("[crossfade] album_match: no match (different dirs)\n"));
 		return 0;
 	}
 
@@ -1032,11 +1074,11 @@ album_match(gchar *old, gchar *new)
 
 	if ((old_track < 0) || (old_track + 1 != new_track))
 	{
-		DEBUG(("[crossfade] album_match: " "no match (same dir, but non-successive (%d, %d))\n", old_track, new_track));
+		DEBUG(("[crossfade] album_match: no match (same dir, but non-successive (%d, %d))\n", old_track, new_track));
 		return 0;
 	}
 
-	DEBUG(("[crossfade] album_match: " "match detected (same dir, successive tracks (%d, %d))\n", old_track, new_track));
+	DEBUG(("[crossfade] album_match: match detected (same dir, successive tracks (%d, %d))\n", old_track, new_track));
 
 	return old_track;
 }
@@ -1045,7 +1087,8 @@ static gint
 xfade_open_audio(AFormat fmt, int rate, int nch)
 {
 	gint pos;
-	gchar *file;
+	gchar *file, *title, *comment;
+	/* id3_t id3; */
 
 	struct timeval tv;
 	glong dt;
@@ -1063,13 +1106,26 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 
 	/* get filename */
 #ifdef HAVE_PLAYLIST_GET_FILENAME
-	pos  = get_playlist_position();     /* XMMS */
-	file = playlist_get_filename(pos);  /* XMMS */
+	pos     = get_playlist_position();      /* XMMS */
+	file    = playlist_get_filename(pos);   /* XMMS */
+	title   = playlist_get_songtitle(pos);  /* XMMS */
+	comment = playlist_get_fadeinfo ? playlist_get_fadeinfo(pos) : NULL;
 #else
-	pos  = xmms_remote_get_playlist_pos (session_id);
-	file = xmms_remote_get_playlist_file(session_id, pos);
+	pos     = xmms_remote_get_playlist_pos  (session_id);
+	file    = xmms_remote_get_playlist_file (session_id, pos);
+	title   = xmms_remote_get_playlist_title(session_id, pos);
+	comment = NULL;
 #endif
 	DEBUG(("[crossfade] open_audio: bname=\"%s\"\n", g_basename(file)));
+	DEBUG(("[crossfade] open_audio: title=\"%s\"\n", title));
+
+	/* try to read comment from ID3 tag */
+#if 0
+	if (!comment && get_id3(file, &id3))
+		comment = g_strdup(id3.comment);
+#endif
+
+	DEBUG(("[crossfade] open_audio: comment=\"%s\"\n", comment));
 
 	/* is this an automatic crossfade? */
 	if (last_filename && (fade_config == &config->fc[FADE_CONFIG_XFADE]))
@@ -1114,8 +1170,8 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 				fade_config = &config->fc[FADE_CONFIG_ALBUM];
 			}
 		}
-		g_free(last_filename);
 	}
+	g_free(last_filename);
 	last_filename = g_strdup(file);
 
 #if 0
@@ -1155,41 +1211,6 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 		else
 			DEBUG(("[crossfade] open_audio: no quantaudio comment found\n"));
 
-#if 0
-		/* configure fade_config with fadeout from last song and fadein from new song */
-		if (fade_config)
-		{
-			config->fc[FADE_CONFIG_TIMING].out_enable = fade_config->out_enable;
-			config->fc[FADE_CONFIG_TIMING].out_volume = fade_config->out_volume;
-			config->fc[FADE_CONFIG_TIMING].out_len_ms = fade_config->out_len_ms;
-			config->fc[FADE_CONFIG_TIMING].ofs_custom_ms = 0;
-			config->fc[FADE_CONFIG_TIMING].in_enable = fade_config->in_enable;
-			config->fc[FADE_CONFIG_TIMING].in_volume = fade_config->in_volume;
-			config->fc[FADE_CONFIG_TIMING].in_len_ms = fade_config->in_len_ms;
-
-			if (have_last_qa && ((fade_config->config == FADE_CONFIG_XFADE) ||
-			                     (fade_config->config == FADE_CONFIG_ALBUM)))
-			{
-				fade_config = &config->fc[FADE_CONFIG_TIMING];
-				fade_config->out_enable = TRUE;
-				fade_config->out_volume = 0;
-				fade_config->out_len_ms = (gint) ((last_qa.length - last_qa.mix_out) * 1000);
-				fade_config->ofs_custom_ms = -fade_config->out_len_ms;	/* FIXME: */
-				DEBUG(("[crossfade] open_audio: CONFIG_TIMING: using last song's fadeout parameter\n"));
-			}
-
-			if (have_qa && ((fade_config->config == FADE_CONFIG_XFADE) ||
-			                (fade_config->config == FADE_CONFIG_ALBUM) ||
-			                (fade_config->config == FADE_CONFIG_MANUAL)))
-			{
-				fade_config = &config->fc[FADE_CONFIG_TIMING];
-				fade_config->in_enable = TRUE;
-				fade_config->in_volume = 0;
-				fade_config->in_len_ms = (gint) (qa.mix_in * 1000);
-				DEBUG(("[crossfade] open_audio: CONFIG_TIMING: using new song's fadein parameter\n"));
-			}
-		}
-#endif
 	}
 	else
 		have_qa = FALSE;
@@ -1199,6 +1220,102 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 		DEBUG(("[crossfade] open_audio: volume normalizer: factor=%.3f\n", volume_context.factor));
 #endif
 
+#ifdef TIMING_COMMENTS
+	last_timing = current_timing;
+	current_timing. in.enable = FALSE;
+	current_timing.out.enable = FALSE;
+	if (comment)
+	{
+		gchar *str;
+		if ((str = strstr(comment, "fadein=")))
+		{
+			current_timing.in.enable =
+			(3 == sscanf(str + 7, "%d,%d,%d",
+				&current_timing.in.len_ms,
+				&current_timing.in.volume,
+				&current_timing.in.skip_ms));
+			current_timing.in.ofs_ms = 0;  /* not used */
+		}
+
+		if ((str = strstr(comment, "fadeout=")))
+		{
+			current_timing.out.enable =
+			(4 == sscanf(str + 8, "%d,%d,%d,%d",
+				&current_timing.out.len_ms,
+				&current_timing.out.volume,
+				&current_timing.out.skip_ms,
+				&current_timing.out.ofs_ms));
+		}
+	}
+
+#if 0
+	//
+	// use the fade info only on a regular, non-manual songchange
+	//
+	if (fade_config && !((fade_config->config == FADE_CONFIG_XFADE) ||
+	                     (fade_config->config == FADE_CONFIG_ALBUM)))
+	      last_timing.out.enable = FALSE;
+#endif
+
+#if 1	
+	if (last_timing.out.enable || current_timing.in.enable)
+#else
+	if ((last_timing.out.enable || current_timing.in.enable) &&
+	   (!fade_config ||
+	    (fade_config->config == FADE_CONFIG_XFADE) ||
+	    (fade_config->config == FADE_CONFIG_ALBUM)))
+#endif
+	{
+		config->fc[FADE_CONFIG_TIMING].out_len_ms    = xfade_cfg_fadeout_len   (fade_config);
+		config->fc[FADE_CONFIG_TIMING].out_volume    = xfade_cfg_fadeout_volume(fade_config);
+		config->fc[FADE_CONFIG_TIMING].out_skip_ms   = 0;
+		config->fc[FADE_CONFIG_TIMING].ofs_custom_ms = xfade_cfg_offset        (fade_config);
+		config->fc[FADE_CONFIG_TIMING].in_skip_ms    = 0;
+		config->fc[FADE_CONFIG_TIMING].in_len_ms     = xfade_cfg_fadein_len    (fade_config);
+		config->fc[FADE_CONFIG_TIMING].in_volume     = xfade_cfg_fadein_volume (fade_config);
+		config->fc[FADE_CONFIG_TIMING].flush         = fade_config && fade_config->flush;
+
+		if (last_timing.out.enable && current_timing.in.enable)
+			config->fc[FADE_CONFIG_TIMING].ofs_custom_ms = 0;
+
+		if (last_timing.out.enable)
+		{
+			DEBUG(("[crossfade] open_audio: TIMING: out: enable=%d len=%d volume=%d skip=%d ofs=%d\n",
+				last_timing.out.enable,
+				last_timing.out.len_ms,
+				last_timing.out.volume,
+				last_timing.out.skip_ms,
+				last_timing.out.ofs_ms));
+
+			config->fc[FADE_CONFIG_TIMING].out_len_ms     = last_timing.out.len_ms;
+			config->fc[FADE_CONFIG_TIMING].out_volume     = last_timing.out.volume;
+			config->fc[FADE_CONFIG_TIMING].out_skip_ms    = last_timing.out.skip_ms;
+			config->fc[FADE_CONFIG_TIMING].ofs_custom_ms += last_timing.out.ofs_ms;
+		}
+		if (current_timing.in.enable)
+		{
+			DEBUG(("[crossfade] open_audio: TIMING:  in: enable=%d len=%d volume=%d skip=%d\n",
+				current_timing.in.enable,
+				current_timing.in.len_ms,
+				current_timing.in.volume,
+				current_timing.in.skip_ms));
+				
+			config->fc[FADE_CONFIG_TIMING].in_skip_ms     = current_timing.in.skip_ms;
+			config->fc[FADE_CONFIG_TIMING].in_len_ms      = current_timing.in.len_ms;
+			config->fc[FADE_CONFIG_TIMING].in_volume      = current_timing.in.volume;
+			config->fc[FADE_CONFIG_TIMING].ofs_custom_ms -= current_timing.in.ofs_ms; 
+			/* NOTE: in.ofs_ms is currently not used always 0 */
+		}
+		
+		fade_config = &config->fc[FADE_CONFIG_TIMING];
+	}
+#endif
+
+	/* cleanup */
+	g_free(file); file = NULL;
+	g_free(title); title = NULL;
+	g_free(comment); comment = NULL;
+
 	/* check for HTTP streaming */
 	if (config->enable_http_workaround && (0 == strncasecmp(file, "http://", 7)))
 	{
@@ -1409,6 +1526,16 @@ xfade_write_audio(void *ptr, int length)
 		length = free;
 	}
 
+	/* skip beginning of song */
+	if ((length > 0) && (buffer->skip > 0))
+	{
+		gint blen = MIN(length, buffer->skip);
+
+		buffer->skip -= blen;
+		length -= blen;
+		ptr += blen;
+	}
+
 	/* kill leading gap */
 	if ((length > 0) && (buffer->gap > 0))
 	{
@@ -1711,13 +1838,22 @@ buffer_thread_f(void *arg)
 				buffer->silence_len = MS2B(100);
 			}
 
-			/* FIXME: make timeout configureable */
 			/* 0.3.9: Check for timeout only if we have not been stopped for restart */
-			if (((timeout < 0) || (timeout >= 10 && !output_restart)) && !input_playing)
+			/* 0.3.11: When using the songchange hack, depend on it's output_restart 
+			 *         flag. Without the hack, use the configuration's dialog songchange
+			 *         timeout setting instead of a fixed timeout value. */
+			if (input_stopped_for_restart && !output_restart)
 			{
 				if (playing)
 					DEBUG(("[crossfade] buffer_thread_f: timeout:"
-					       " input did not restart after %ld ms (playing=%d)\n", timeout, playing));
+					       " stopping after %ld ms (songchange patch)\n", timeout));
+				stopping = TRUE;
+			}
+			else if (((timeout < 0) || (timeout >= config->songchange_timeout && !output_restart)) && !input_playing)
+			{
+				if (playing)
+					DEBUG(("[crossfade] buffer_thread_f: timeout:"
+					       " input did not restart after %ld ms\n", timeout));
 				stopping = TRUE;
 			}
 		}
@@ -1837,7 +1973,7 @@ buffer_thread_f(void *arg)
 
 			if (output_flush_time != output_time)
 			{
-#ifdef SONGCHANGE_TIMEOUT
+#if 0  // defined(SONGCHANGE_TIMEOUT)
 				if (timeout < config->songchange_timeout)
 				{
 					/* slow down output, but always write _some_ data */
@@ -1927,7 +2063,7 @@ buffer_thread_f(void *arg)
 				length = op_free;
 
 			/* HACK: throttle output (used with fast output plugins) */
-#ifdef SONGCHANGE_TIMEOUT
+#if 0  // defined(SONGCHANGE_TIMEOUT)
 			if (the_op_config.throttle_enable && !realtime && opened && (timeout < config->songchange_timeout))
 #else
 			if (the_op_config.throttle_enable && !realtime && opened)
@@ -2172,10 +2308,14 @@ xfade_close_audio()
 		 * NOTE: Not quite. There still are some problems when
 		 *       XMMS is exitting while a song is playing. So
 		 *       this HACK has been enabled again.
+		 *
+		 * NOTE: Another thing: If output_keep_opened is enabled,
+		 *       close_audio() is never called, so that the patch
+		 *       can not work.
 		 */
 #if 1
-		if ((xmms_playlist_get_info_going && !*xmms_playlist_get_info_going)
-		 || (xmms_is_quitting && *xmms_is_quitting))
+		if ((xmms_is_quitting && *xmms_is_quitting)
+		 || (xmms_playlist_get_info_going && !*xmms_playlist_get_info_going))
 		{
 			DEBUG(("[crossfade] close: stop (about to quit)\n"))
 
@@ -2203,6 +2343,7 @@ xfade_close_audio()
 	}
 	else
 	{
+		/* gint x = *((gint *)0); */  /* force SEGFAULT for debugging */
 		DEBUG(("[crossfade] close: songchange/eop\n"));
 
 		/* kill trailing gap (does not use buffer->gap_*) */
@@ -2212,7 +2353,7 @@ xfade_close_audio()
 			gint gap_level = xfade_cfg_gap_trail_level(config);
 			gint length    = MIN(gap_len, buffer->used);
 
-			DEBUG(("[crossfade] close: len=%d level=%d length=%d\n", gap_len, gap_level, length));
+			/* DEBUG(("[crossfade] close: len=%d level=%d length=%d\n", gap_len, gap_level, length)); */
 
 			buffer->gap_killed = 0;
 			while (length > 0)
@@ -2294,8 +2435,32 @@ xfade_close_audio()
 void
 xfade_flush(gint time)
 {
+	gint pos;
+	gchar *file;
+
 	DEBUG(("[crossfade] flush: time=%d\n", time));
 
+	/* get filename */
+#ifdef HAVE_PLAYLIST_GET_FILENAME
+	pos     = get_playlist_position();      /* XMMS */
+	file    = playlist_get_filename(pos);   /* XMMS */
+#else
+	pos     = xmms_remote_get_playlist_pos  (session_id);
+	file    = xmms_remote_get_playlist_file (session_id, pos);
+#endif
+
+#if defined(COMPILE_FOR_AUDACIOUS)
+	/* HACK: special handling for audacious, which only calls flush(0) on a songchange */
+	if (file && last_filename && strcmp(file, last_filename) != 0)
+	{
+		DEBUG(("[crossfade] flush: filename changed, forcing close/reopen...\n"));
+		xfade_close_audio();
+		xfade_open_audio(in_format.fmt, in_format.rate, in_format.nch);
+		DEBUG(("[crossfade] flush: filename changed, forcing close/reopen... done\n"));
+		return;
+	}
+#endif
+
 	/* lock buffer */
 	pthread_mutex_lock(&buffer_mutex);
 
@@ -2567,3 +2732,31 @@ xfade_output_time()
 #endif
 	return time;
 }
+
+void
+xfade_cleanup()
+{
+	DEBUG(("[crossfade] cleanup:\n"));
+
+	/* lock buffer */
+	pthread_mutex_lock(&buffer_mutex);
+	
+	/* check if buffer thread is still running */
+	if (output_opened)
+	{
+		DEBUG(("[crossfade] cleanup: closing output\n"));
+		
+		stopped = TRUE;
+		
+		/* wait for buffer thread to clean up and terminate */
+		pthread_mutex_unlock(&buffer_mutex);
+		if (pthread_join(buffer_thread, NULL))
+			PERROR("[crossfade] cleanup: phtread_join()");
+		pthread_mutex_lock(&buffer_mutex);
+	}
+	
+	/* unlock buffer */
+	pthread_mutex_unlock(&buffer_mutex);
+		
+	DEBUG(("[crossfade] cleanup: done\n"));
+}
