@@ -29,37 +29,42 @@
 #endif
 
 #include <pthread.h>
-#include <gdk/gdk.h>
-#include <gtk/gtk.h>
+#include <glib.h>
 
-#ifdef COMPILE_FOR_XMMS
+#if defined(COMPILE_FOR_XMMS) || defined(COMPILE_FOR_NOXMMS)
 #  include <xmms/plugin.h>
 #  include <xmms/xmmsctrl.h>
 #  include <xmms/configfile.h>
 #  include <xmms/util.h>
 #endif
-#ifdef COMPILE_FOR_BMP
+#if defined(COMPILE_FOR_BMP)
 #  include <bmp/plugin.h>
 #  include <bmp/beepctrl.h>
 #  include <bmp/configfile.h>
 #  include <bmp/util.h>
 #endif
-#ifdef COMPILE_FOR_AUDACIOUS
+#if defined(COMPILE_FOR_AUDACIOUS)
 #  include <audacious/plugin.h>
 #  include <audacious/beepctrl.h>
-#  include <audacious/configfile.h>
+#  include <audacious/configdb.h>
 #  include <audacious/util.h>
 #endif
 
 #include "debug.h"
 
 #undef  VOLUME_NORMALIZER
-#undef  SONGCHANGE_TIMEOUT
+#define TIMING_COMMENTS
 
 #define OUTPUT_RATE the_rate
 #define OUTPUT_NCH  2
 #define OUTPUT_BPS (OUTPUT_RATE * OUTPUT_NCH * 2)
 
+#if defined(HAVE_LIBSAMPLERATE)
+#  define MAX_RATE 192000
+#else
+#  define MAX_RATE 48000
+#endif
+
 #define SYNC_OUTPUT_TIMEOUT 2000
 
 #define OUTPUT_METHOD_BUILTIN_OSS  0
@@ -292,20 +297,22 @@
       TYPEMASK_PAUSE            /* type_mask */				\
     },									\
     { FADE_CONFIG_TIMING,       /* config */				\
-      FADE_TYPE_NONE,           /* type */				\
+      FADE_TYPE_ADVANCED_XF,    /* type */				\
       0,                        /* - */					\
       0,                        /* - */					\
-      TRUE, 0, 0,               /* out_enable, _len_ms, - */		\
+      TRUE, 0, 0,               /* out_enable, _len_ms, _volume */	\
       FC_OFFSET_CUSTOM,         /* - */					\
       FC_OFFSET_CUSTOM, 0,      /* - ofs_custom_ms */			\
       FALSE,                    /* - */					\
-      TRUE, 0, 0,               /* in_enable, _len_ms, - */		\
+      TRUE, 0, 0,               /* in_enable, _len_ms, _volume */	\
       FALSE,                    /* - */					\
       0,                        /* - */					\
       FALSE,                    /* - */					\
       0, 0,                     /* - */					\
       FALSE,                    /* flush */				\
-      TYPEMASK_TIMING           /* type_mask */				\
+      TYPEMASK_TIMING,          /* type_mask */				\
+      500,                      /* out_skip_ms */			\
+      500                       /* in_skip_ms */			\
     }									\
   },									\
   TRUE,                         /* gap_lead_enable */			\
@@ -359,6 +366,7 @@ gint playlist_get_current_length();
 #ifdef HAVE_PLAYLIST_GET_FILENAME
 gchar *playlist_get_filename(gint pos);
 #endif
+gchar *playlist_get_songtitle(gint pos);
 
 /* config change callbacks */
 void xfade_realize_config();
@@ -400,6 +408,9 @@ typedef struct
 	gint gap_killed;	/* number of bytes that were killed last time */
 	gint gap_skipped;	/* negative/positive samples skipped */
 
+	gint skip;		/* > 0: skipping beginning of song */
+	gint skip_len;		/* total number of samples to skip */
+
 	/* ---------------------------------------------------------------------- */
 
 	gint silence;		/* > 0: delay until start of silence */
@@ -443,6 +454,10 @@ typedef struct
 	/* additional stuff which is not configureable by the user */
 	gboolean flush;		// TRUE for manual, FALSE for xfade config
 	guint32 type_mask;	// bitmask for FADE_TYPEs
+	
+	/* new fields as of v0.3.11: skip beginning/end of song */
+	gint out_skip_ms;
+	gint in_skip_ms;
 }
 fade_config_t;
 
