@@ -1,6 +1,6 @@
 /*
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>
+ *  Copyright (C) 2000-2005  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -24,11 +24,12 @@
 #include <stdio.h>
 #include <stdarg.h>
 
-void debug(char *format, ...)
+void
+debug(char *format, ...)
 {
-  va_list ap;
+	va_list ap;
 
-  va_start(ap, format);
-  vfprintf(stderr, format, ap);
-  va_end  (ap);
+	va_start(ap, format);
+	vfprintf(stderr, format, ap);
+	va_end(ap);
 }
