@@ -80,7 +80,7 @@ setup_format(AFormat fmt, gint rate, gint nch, format_t * format)
 	}
 
 	/* check rate */
-	if ((rate < 1) || (rate > 65535))
+	if ((rate < 1) || (rate > MAX_RATE))
 	{
 		DEBUG(("[crossfade] setup_format: illegal rate (%d)!\n", rate));
 		return -1;
