@@ -29,6 +29,8 @@
 #include <gtk/gtk.h>
 
 #include "monitor.h"
+#include "configure.h"
+#include "cfgutil.h"
 #include "crossfade.h"
 
 #ifdef HAVE_GTK_2_0
@@ -174,6 +176,7 @@ xfade_check_monitor_win()
 		gtk_signal_connect(GTK_OBJECT(monitor_win), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &monitor_win);
 
 		/* show window */
+		gtk_widget_hide(GTK_WIDGET(lookup_widget(monitor_win, "monitor_seekeof_button")));
 		gtk_widget_show(monitor_win);
 
 		/* get pointers to widgets (used by crossfade.c to update the monitor) */
@@ -435,3 +438,25 @@ xfade_stop_monitor()
 	gtk_timeout_remove(monitor_tag);
 	monitor_active = FALSE;
 }
+
+#if defined(HAVE_INPUT_SEEK)
+void input_seek(int time);  /* XMMS */
+void
+on_monitor_seekeof_button_clicked(GtkButton *button, gpointer user_data)
+{  
+	gint total    = playlist_get_current_length();  /* XMMS */
+	gint offset   = xfade_cfg_offset(&config->fc[FADE_CONFIG_XFADE])
+	              - config->songchange_timeout;
+	gint position = total + offset - 2500;
+
+	if (position < 0)
+		return;
+
+	DEBUG(("[crossfade] monitor_seek_eof: total=%d offset=%d position=%d\n", total, offset, position))
+	
+	input_seek(position/1000);  /* XMMS */
+}
+#else
+void on_monitor_seekeof_button_clicked(GtkButton *button, gpointer user_data)
+{ }
+#endif
