@@ -67,7 +67,10 @@ static OutputPlugin oss_op = {
 	NULL,
 	"Builtin OSS Driver",
 	NULL,
+#if defined (COMPILE_FOR_AUDACIOUS)
 	NULL,
+#endif
+	NULL,	
 	NULL,
 	oss_get_volume,
 	oss_set_volume,
@@ -80,6 +83,9 @@ static OutputPlugin oss_op = {
 	oss_buffer_playing,
 	oss_output_time,
 	oss_written_time,
+#if defined(COMPILE_FOR_AUDACIOUS)
+	NULL,
+#endif
 };
 
 /* local prototypes */
@@ -311,7 +317,7 @@ oss_open_audio(AFormat fmt, int rate, int nch)
 	}
 
 	/* check rate */
-	if ((rate < 1) || (rate > 65535))
+	if ((rate < 1) || (rate > MAX_RATE))
 	{
 		DEBUG(("[xfade-oss] open_audio: illegal rate=%d!\n", rate));
 		return 0;
