@@ -80,16 +80,16 @@ lcm(gint i, gint j)
 
 
 void
-rate_init(rate_context_t * rc)
+rate_init(rate_context_t *rc)
 {
 	memset(rc, 0, sizeof(*rc));
 	volume_init(&rc->vc);
 }
 
 void
-rate_config(rate_context_t * rc, gint in_rate, gint out_rate, int converter_type)
+rate_config(rate_context_t *rc, gint in_rate, gint out_rate, int converter_type)
 {
-	if ((in_rate <= 0) || (in_rate > 65535) || (out_rate <= 0) || (out_rate > 65535))
+	if ((in_rate <= 0) || (in_rate > MAX_RATE) || (out_rate <= 0) || (out_rate > MAX_RATE))
 	{
 		DEBUG(("[crossfade] rate_config: " "illegal rates (in=%d, out=%d)!\n", in_rate, out_rate));
 		return;
@@ -101,24 +101,26 @@ rate_config(rate_context_t * rc, gint in_rate, gint out_rate, int converter_type
 	rc->out_rate = out_rate;
 #ifdef HAVE_LIBSAMPLERATE
 	rc->converter_type = converter_type;
-	rc->src_data.src_ratio = out_rate * 1.0 / in_rate;
+	rc->src_data.src_ratio = (double)out_rate / in_rate;
 	if (!(rc->src_state = src_new(converter_type, 2, NULL)))
 	{
 		DEBUG(("[crossfade] rate_config: error creating converter!\n"));
 		return;
 	}
-	DEBUG(("[crossfade] rate_config: using \"%s\" for %d -> %d rate conversion\n",
-	       src_get_name(converter_type), in_rate, out_rate));
+	
+	if (in_rate != out_rate)
+		DEBUG(("[crossfade] rate_config: using \"%s\" for %d -> %d rate conversion\n",
+		       src_get_name(converter_type), in_rate, out_rate));
 
 	src_set_ratio(rc->src_state, rc->src_data.src_ratio);
 	rc->src_data.end_of_input = 0;
 #else
 	rc->lcm_rate = lcm(in_rate, out_rate);
-	rc->in_skip = rc->lcm_rate / in_rate;
+	rc-> in_skip = rc->lcm_rate /  in_rate;
 	rc->out_skip = rc->lcm_rate / out_rate;
-	rc->in_ofs = 0;
-	rc->out_ofs = 0;
-	rc->started = FALSE;
+	rc-> in_ofs  = 0;
+	rc->out_ofs  = 0;
+	rc->started  = FALSE;
 #endif
 	rc->valid = TRUE;
 }
@@ -163,14 +165,24 @@ final_quantize(volume_context_t * vc, gfloat v, gfloat scale)
 }
 
 gint
-rate_flow(rate_context_t * rc, gpointer * buffer, gint length)
+rate_flow(rate_context_t *rc, gpointer * buffer, gint length)
 {
 	gint size;
 	gfloat volume_scale_l = 1, volume_scale_r = 1;
 	struct timeval tv;
 	glong dt;
 
-	/* print clipping warnings at most once every second : */
+#ifdef HAVE_LIBSAMPLERATE
+	gint16 *int_p;
+	gfloat *float_p;
+	gint i, out_bound, out_len, error = 0;
+#else  /* rustic resampling */
+	gpointer data;
+	gint isamp, emitted = 0;
+	gint16 *out, *in = *buffer;
+#endif
+
+	/* print clipping warnings at most once every second: */
 	gettimeofday(&tv, NULL);
 	dt = (tv.tv_sec - rc->vc.tv_last.tv_sec) * 1000 + (tv.tv_usec - rc->vc.tv_last.tv_usec) / 1000;
 	if (((dt < 0) || (dt > 1000)) && (rc->vc.clips > 0))
@@ -183,7 +195,7 @@ rate_flow(rate_context_t * rc, gpointer * buffer, gint length)
 	/* attenuate 1dB every 4% -> 25dB for 100% */
 	if (config->mixer_software)
 	{
-		volume_scale_l = volume_compute_factor(config->mixer_vol_left, 25);
+		volume_scale_l = volume_compute_factor(config->mixer_vol_left,  25);
 		volume_scale_r = volume_compute_factor(config->mixer_vol_right, 25);
 	}
 
@@ -193,7 +205,7 @@ rate_flow(rate_context_t * rc, gpointer * buffer, gint length)
 		gint16 *int_p = *buffer, *int_p2 = NULL;
 		gint i;
 
-		gint error = realloc_if_needed((gpointer *) & rc->data, &rc->size, length);
+		gint error = realloc_if_needed((gpointer *) &rc->data, &rc->size, length);
 		assert(error != -1);
 		int_p2 = rc->data;
 
@@ -211,10 +223,6 @@ rate_flow(rate_context_t * rc, gpointer * buffer, gint length)
 	}
 
 #ifdef HAVE_LIBSAMPLERATE
-	gint16 *int_p;
-	gfloat *float_p;
-	gint i, out_bound, out_len, error = 0;
-
 	assert(length % 4 == 0);
 	length /= 4;		/* 2 channels x 2 bytes */
 	/* safe upper bound of output length: */
@@ -238,14 +246,14 @@ rate_flow(rate_context_t * rc, gpointer * buffer, gint length)
 	error = realloc_if_needed((gpointer *) & rc->data, &rc->size, size);
 	assert(error != -1);
 
-	/* putting data into the float buffers : */
+	/* putting data into the float buffers: */
 	float_p = rc->src_data.data_in;
 	int_p = *buffer;
 	for (i = 0; i < 2 * length; i++)
 		*(float_p++) = (gfloat) * (int_p++) / 32768.;
 	assert(float_p == rc->src_data.data_in + length * 2);
 
-	/* process the float buffers : */
+	/* process the float buffers: */
 	error = src_process(rc->src_state, &rc->src_data);
 	if (error)
 		DEBUG(("[crossfade] rate_flow: src_error %d (%s)\n", error, src_strerror(error)));
@@ -260,7 +268,6 @@ rate_flow(rate_context_t * rc, gpointer * buffer, gint length)
 	{
 		gfloat v_l = 32768 * *(float_p++);
 		gfloat v_r = 32768 * *(float_p++);
-
 		*(int_p++) = final_quantize(&rc->vc, v_l, volume_scale_l);
 		*(int_p++) = final_quantize(&rc->vc, v_r, volume_scale_r);
 	}
@@ -270,10 +277,6 @@ rate_flow(rate_context_t * rc, gpointer * buffer, gint length)
 
 #else  /* rustic resampling */
 
-	gpointer data;
-	gint isamp, emitted = 0;
-	gint16 *out, *in = *buffer;
-
 	/* some sanity checks */
 	if (length & 3)
 	{
@@ -354,7 +357,7 @@ rate_flow(rate_context_t * rc, gpointer * buffer, gint length)
 }
 
 void
-rate_free(rate_context_t * rc)
+rate_free(rate_context_t *rc)
 {
 #ifdef HAVE_LIBSAMPLERATE
 	if (rc->src_state)
