@@ -90,11 +90,13 @@ volume_set_song_rms(volume_context_t * vc, gint song_rms)
 gfloat
 volume_compute_factor(gint percent, gint dB_range)
 {
+	gfloat dB;
+
 	if (percent >= 100)
 		return 1;
 	if (percent <= 0)
 		return 0;
-	gfloat dB = (percent - 100) / 100.0 * dB_range;
+	dB = (percent - 100) / 100.0 * dB_range;
 	return pow(10, dB / 20);
 }
 
