@@ -1,6 +1,6 @@
 /*
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>
+ *  Copyright (C) 2000-2005  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -35,24 +35,26 @@
 
 typedef struct
 {
-  gboolean active;
-  gint     target_rms;
-  gint     song_rms;
-  gfloat   factor;
+	gboolean active;
+	gint target_rms;
+	gint song_rms;
+	gfloat factor;
 
-  struct timeval tv_last;
-  gint clips;
+	struct timeval tv_last;
+	gint clips;
 }
 volume_context_t;
 
-void volume_init(volume_context_t *cc);
-void volume_flow(volume_context_t *cc, gpointer *buffer, gint length);
-void volume_free(volume_context_t *cc);
+void volume_init(volume_context_t * cc);
+void volume_flow(volume_context_t * cc, gpointer * buffer, gint length);
+void volume_free(volume_context_t * cc);
 
-void volume_set_active    (volume_context_t *cc, gboolean active);
-void volume_set_target_rms(volume_context_t *cc, gint target_rms);
-void volume_set_song_rms  (volume_context_t *cc, gint song_rms);
+void volume_set_active(volume_context_t * cc, gboolean active);
+void volume_set_target_rms(volume_context_t * cc, gint target_rms);
+void volume_set_song_rms(volume_context_t * cc, gint song_rms);
 
-void volume_apply(gpointer *buffer, gint length, gfloat left, gfloat right);
+gfloat volume_compute_factor(gint percent, gint dB_range);	/*compute factor corresponding to
+								   attenuations in [- dB_range, 0]
+								 */
 
-#endif  /* _VOLUME_H_ */
+#endif /* _VOLUME_H_ */
